; ==============================================================================
;	MGSDRVȈՃvC[
; ==============================================================================

		include		"MGSDRV_API.ASM"

bdos	:=			0x0005
dma		:=			0x0080
extbio	:=			0xFFCA
enaslt	:=			0x0024

		org			0x0100

entry::
		ld			[save_sp], sp

		; show version message
		ld			c, 0x09
		ld			de, msg_mgsp_ver
		call		bdos

		; check installed MGSDRV
		call		check_mgsdrv
		ld			de, msg_error_mgsdrv_not_found
		jp			nz, error_exit

		push		bc
		ld			de, mgs_jump_table
		ld			bc, 3 * 5
		ldir									; copy jump table

		pop			hl
		ld			de, mgs_parameter_block
		ld			bc, 21
		ldir									; copy parameter block

		; get memory mapper support routine
		ld			de, 0x0402
		call		extbio

		ld			de, mapper_jump_table
		ld			bc, 16 * 3
		ldir

		; get current segment on page1
		call		mapper_get_p1
		ld			[mapper_segment_p1], a
		call		mapper_get_p2
		ld			[mapper_segment_p2], a

		; change slot of page1 (mgsdrv body)
		ld			h, 0x40
		ld			a, [mgs_id_drvslt]
		call		enaslt

		; change slot of page2 (music data)
		ld			h, 0x80
		ld			a, [mgs_id_datslt]
		call		enaslt

		; change mapper segment of page1
		ld			a, [mgs_id_drvseg]
		call		mapper_put_p1

		; change mapper segment of page2
		ld			a, [mgs_id_datseg]
		call		mapper_put_p2

		; show MGSDRV version
		ld			c, 0x09
		ld			de, msg_detect
		call		bdos

		ld			a, [mgs_id_mgsver + 1]
		ld			hl, msg_1byte_hex
		call		dec_to_hex

		ld			c, 0x09
		ld			de, msg_1byte_hex + 1
		call		bdos

		ld			c, 0x02
		ld			e, '.'
		call		bdos

		ld			a, [mgs_id_mgsver]
		ld			hl, msg_1byte_hex
		call		dec_to_hex

		ld			c, 0x09
		ld			de, msg_1byte_hex
		call		bdos

		ld			c, 0x09
		ld			de, msg_ret
		call		bdos

		; show OPLL slot number
		ld			c, 0x09
		ld			de, msg_opll_slot_num
		call		bdos
		ld			ix, [mgs_id_mibadr]
		ld			a, [ix + 0]
		call		put_slot_num
		ld			c, 0x09
		ld			de, msg_ret
		call		bdos

		; show SCC slot number
		ld			c, 0x09
		ld			de, msg_scc_slot_num
		call		bdos
		ld			ix, [mgs_id_mibadr]
		ld			a, [ix + 1]
		call		put_slot_num
		ld			c, 0x09
		ld			de, msg_ret
		call		bdos

		; check command line options
		call		check_options

		; process options
		call		show_help
		call		load_file
		call		play_music
		call		stop_music

		call		restore_slot

		ld			b, 0
		ld			c, 0x62
		jp			bdos

; ==============================================================================
;	G[I
; ==============================================================================
		scope		error_exit
error_exit::
		ei
		ld			c, 0x09
		call		bdos

		call		restore_slot

		ld			b, 0
		ld			c, 0x62
		jp			bdos
		endscope

; ==============================================================================
;	Xbg\ɖ߂
; ==============================================================================
		scope		restore_slot
restore_slot::
		; change slot of page1 (mgsdrv body)
		ld			h, 0x40
		ld			a, [0xF342]
		call		enaslt

		; change slot of page2 (music data)
		ld			h, 0x80
		ld			a, [0xF343]
		call		enaslt

		; change mapper segment of page1
		ld			a, [mapper_segment_p1]
		call		mapper_put_p1

		; change mapper segment of page2
		ld			a, [mapper_segment_p2]
		call		mapper_put_p2
		ei
		ret
		endscope

; ==============================================================================
;	wv\
; ==============================================================================
		scope		show_help
show_help::
		ld			a, [option_h]
		or			a, a
		jr			z, check_no_options
show:
		ld			c, 0x09
		ld			de, msg_help
		call		bdos
		ret

check_no_options:
		ld			a, [option_p]
		ld			b, a
		ld			a, [option_s]
		or			a, b
		ld			b, a
		ld			a, [target_file_name]
		or			a, b
		ret			nz
		jp			show
		endscope

; ==============================================================================
;	t@Cǂݍ
; ==============================================================================
		scope		load_file
load_file::
		ld			a, [target_file_name]
		or			a, a
		ret			z

		; music stop
		call		mgs_initm

		; open the file
		ld			c, 0x43
		ld			a, 1
		ld			de, target_file_name
		call		bdos
		ld			a, b
		ld			[file_handle], a
		ld			de, msg_cannot_open_file
		jp			nz, error_exit

		; read the file
		ld			c, 0x48
		ld			de, 0x8000
		ld			hl, 0x4000
		call		bdos

		; close the file
		ld			c, 0x45
		ld			a, [file_handle]
		ld			b, a
		call		bdos
		ret
		endscope

; ==============================================================================
;	ȂĐ
; ==============================================================================
		scope		play_music
play_music::
		ld			a, [option_p]
		or			a, a
		ret			z

		; music stop
		call		mgs_initm

		; check music data
		ld			hl, 0x8000
		call		mgs_datck
		ld			de, msg_invalid_data
		jp			nz, error_exit

		; music start
		ld			b, -1
		ld			hl, -1
		ld			de, 0x8000
		call		mgs_plyst
		ei
		ret
		endscope

; ==============================================================================
;	Ȃ~
; ==============================================================================
		scope		stop_music
stop_music::
		ld			a, [option_s]
		or			a, a
		ret			z

		; music stop
		call		mgs_initm
		ret
		endscope

; ==============================================================================
;	R}hCIvV𒲂ׂ
; ==============================================================================
		scope		check_options
check_options::
		ld			hl, dma + 1
		ld			b, 127				; max length of dma
skip_white_space:
		call		get_one
		ret			z
		cp			a, ' '
		jr			z, skip_white_space

		cp			a, '/'				; this is a option?
		jp			z, check_name

		; This is a file name
		ld			de, target_file_name
		ld			c, a
		ld			a, [de]
		or			a, a
		ld			a, c
		jr			nz, skip_2nd_file_name	; 2ڈȍ~̃t@C͖
copy_file_name:
		ld			[de], a
		inc			de
		call		get_one
		ret			z
		cp			a, ' '
		jr			nz, copy_file_name
		jp			skip_white_space
skip_2nd_file_name:
		call		get_one
		ret			z
		cp			a, ' '
		jr			nz, skip_2nd_file_name
		jp			skip_white_space

check_name:
		call		get_upper_one
		ret			z

		cp			a, 'H'
		jr			nz, not_help_option
		ld			a, 1
		ld			[option_h], a
		call		get_upper_one
		ret			z
not_help_option:

		cp			a, 'P'
		jr			nz, not_play_option
		ld			a, 1
		ld			[option_p], a
		call		get_upper_one
		ret			z
not_play_option:

		cp			a, 'S'
		jr			nz, not_stop_option
		ld			a, 1
		ld			[option_s], a
		call		get_upper_one
		ret			z
not_stop_option:

		cp			a, ' '
		jp			z, skip_white_space

		di
		ld			sp, [save_sp]
		ld			de, msg_invalid_option
		jp			error_exit
		endscope

		scope		get_one
get_one::
		ld			a, b
		or			a, a
		ret			z
		ld			a, [hl]
		inc			hl
		dec			b
		or			a, a
		ret
		endscope

		scope		get_upper_one
get_upper_one::
		call		get_one
		ret			z
		cp			a, 'a'
		ret			c
		cp			a, 'z' + 1
		ret			nc
		add			a, 'A' - 'a'
		ret
		endscope

; ==============================================================================
;	(HL)AɓĂl16i_v
; ==============================================================================
		scope		dec_to_hex
dec_to_hex::
		ld			b, a
		rrca
		rrca
		rrca
		rrca
		and			a, 0x0F
		add			a, '0'
		cp			a, '9' + 1
		jr			c, skip1
		add			a, 'A' - '0' - 10
skip1:
		ld			[hl], a
		inc			hl
		ld			a, b
		and			a, 0x0F
		add			a, '0'
		cp			a, '9' + 1
		jr			c, skip2
		add			a, 'A' - '0' - 10
skip2:
		ld			[hl], a
		inc			hl
		ret
		endscope

; ==============================================================================
;	AɓĂXbgԍ\
; ==============================================================================
		scope		put_slot_num
put_slot_num::
		ld			[save_slot], a
		inc			a
		jr			z, put_none
		dec			a
		and			a, 3
		add			a, '0'
		ld			e, a
		ld			c, 0x02
		call		bdos
		ld			a, [save_slot]
		or			a, a
		ret			p

		ld			e, '-'
		ld			c, 0x02
		call		bdos

		ld			a, [save_slot]
		rrca
		rrca
		and			a, 3
		add			a, '0'
		ld			e, a
		ld			c, 0x02
		call		bdos
		ret
put_none:
		ld			c, 0x09
		ld			de, msg_none
		call		bdos
		ret

save_slot:
		db			0
		endscope

; ==============================================================================
;	MGSDRV̏풓`FbN
;	output)
;		Z flag ... 0: MGSDRV, 1: MGSDRVL
; ==============================================================================
		scope		check_mgsdrv
check_mgsdrv::
		xor			a, a
		ld			de, 0xf000			; D=device ID, e=00h
		call		extbio				; call extended BIOS
		inc			a
		ret
		endscope

; ==============================================================================
;	DATA AREA
; ==============================================================================
target_file_name::
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

file_handle::
		db			0
option_h:
		db			0
option_p:
		db			0
option_s:
		db			0
save_sp::
		dw			0

msg_mgsp_ver::
		ds			"MGSP ver1.0"
		db			0x0d, 0x0a
		ds			"Programmed by t.hara."
		db			0x0d, 0x0a, '$'
msg_help::
		ds			">MGSP [options] [MGS file name]"
		db			0x0d, 0x0a
		ds			"/P ... Play music."
		db			0x0d, 0x0a
		ds			"/S ... Stop music."
		db			0x0d, 0x0a
		ds			"/H ... Show this help."
		db			0x0d, 0x0a, '$'
msg_error_mgsdrv_not_found::
		ds			"MGSDRV is not installed."
msg_ret::
		db			0x0d, 0x0a, '$'
msg_invalid_option:
		ds			"Invalid option."
		db			0x0d, 0x0a, '$'
msg_segment_error:
		ds			"Segment error."
		db			0x0d, 0x0a, '$'
msg_detect::
		ds			"Detect MGSDRV $"
msg_opll_slot_num::
		ds			"  OPLL SLOT #$"
msg_scc_slot_num::
		ds			"  SCC  SLOT #$"
msg_none::
		ds			"none$"
msg_2byte_hex::
		ds			"00"
msg_1byte_hex::
		ds			"00$"
msg_cannot_open_file::
		ds			"Cannot open the file."
		db			0x0d, 0x0a, '$'
msg_invalid_data::
		ds			"Invalid music data."
		db			0x0d, 0x0a, '$'
msg_not_enough_memory::
		ds			"Not enough memory."
		db			0x0d, 0x0a, '$'
mapper_segment_p1:
		db			0
mapper_segment_p2:
		db			0
mgs_parameter_block::
mgs_id_drvslt::			;MGSDRV{̂̃Xbg
		db			0
mgs_id_drvseg::			;MGSDRV{̂̃ZOg
		db			0
mgs_id_datslt::			;tf[^̈̃Xbg
		db			0
mgs_id_datseg::			;tf[^̈̃ZOg
		db			0
mgs_id_drvsiz::			;MGSDRV{̂̃TCY
		dw			0
mgs_id_chfomm::			;؂ւp̃Xbg(C)
		db			0
mgs_id_chfome::			;؂ւp̃Xbg(g)
		db			0
mgs_id_mgsver::			;MGSDRṼo[W
		dw			0
mgs_id_mibadr::			;MIBւ̃AhX
		dw			0
mgs_id_trwadr::			;gbN[ÑAhX
		dw			0
mgs_id_trwsiz::			;gbN[ÑTCY
		dw			0
mgs_id_keyasw::			;L[荞ݏtO
		db			0
mgs_id_segmax::			;mۂȃf[^ZOg̐
		db			0
mgs_id_segnum::			;I𒆂̋ȃf[^ԍ
		db			0
mgs_id_segtab::			;mۂȃf[^ZOg̃e[u
		db			0


mgs_jump_table::
mgs_ij_interv::
		db			0xc9, 0xc9, 0xc9
mgs_ij_chgdrv::
		db			0xc9, 0xc9, 0xc9
mgs_ij_revdrv::
		db			0xc9, 0xc9, 0xc9
mgs_ij_calmgs::
		db			0xc9, 0xc9, 0xc9
mgs_ij_chgseg::
		db			0xc9, 0xc9, 0xc9
mapper_jump_table::
mapper_all_seg::	; +00h
		db			0xc9, 0xc9, 0xc9
mapper_fre_seg::	; +03h
		db			0xc9, 0xc9, 0xc9
mapper_rd_seg::		; +06h
		db			0xc9, 0xc9, 0xc9
mapper_wr_seg::		; +09h
		db			0xc9, 0xc9, 0xc9
mapper_cal_seg::	; +0Ch
		db			0xc9, 0xc9, 0xc9
mapper_calls::		; +0Fh
		db			0xc9, 0xc9, 0xc9
mapper_put_ph::		; +12h
		db			0xc9, 0xc9, 0xc9
mapper_get_ph::		; +15h
		db			0xc9, 0xc9, 0xc9
mapper_put_p0::		; +18h
		db			0xc9, 0xc9, 0xc9
mapper_get_p0::		; +1Bh
		db			0xc9, 0xc9, 0xc9
mapper_put_p1::		; +1Eh
		db			0xc9, 0xc9, 0xc9
mapper_get_p1::		; +21h
		db			0xc9, 0xc9, 0xc9
mapper_put_p2::		; +24h
		db			0xc9, 0xc9, 0xc9
mapper_get_p2::		; +27h
		db			0xc9, 0xc9, 0xc9
mapper_put_p3::		; +2Ah
		db			0xc9, 0xc9, 0xc9
mapper_get_p3::		; +2Dh
		db			0xc9, 0xc9, 0xc9
